window.addEventListener("load", XPathDOM_initialize, false);

dump("chrome://xpath/content/dom.js loaded\n");

function XPathDOM_initialize() {
  dump("XPathDOM_initialize()\n");
  viewer.initXPath();
}

DOMViewer.prototype.initXPath =
  function(node)
  {
    dump("DOMViewer.initXPath()\n");
    // I hope all this reference shuffling works!
    this.xOldDOMView = this.mDOMView;
    this.xDOMView = new xpathTreeView(this.xOldDOMView);
    this.mDOMView = this.xDOMView;
    this.mDOMTree.treeBoxObject.view = this.xDOMView;
  };

DOMViewer.prototype.xSelectNode =
  function(node)
  {
    this.mDOMView.performActionOnNode( "xpath:select", node );
  };

DOMViewer.prototype.xClearSelection =
  function()
  {
    this.mDOMView.performAction( "xpath:clear" );
  };

const AtomServiceCID = "@mozilla.org/atom-service;1";

function xpathTreeView(view) {
	this.domview = view
}
xpathTreeView.prototype = {
	domview : null,
	selectedNodes : [], // Indexes selected nodes, so that they can be cleared later.
	treebox : null,
	get rowCount() {
		return this.domview.rowCount;
	},
	get rootNode() {
		return this.domview.rootNode;
	},
	set rootNode(value) {
		this.performAction("xpath:clear");
		return this.domview.rootNode = value;
	},
	get selection() {
		return this.domview.selection;
	},
	set selection(value) {
		return this.domview.selection = value;
	},
	get showAnonymousContent() {
		return this.domview.showAnonymousContent;
	},
	set showAnonymousContent(value) {
		return this.domview.showAnonymousContent = value;
	},
	get showSubDocuments() {
		return this.domview.showSubDocuments;
	},
	set showSubDocuments(value) {
		return this.domview.showSubDocuments = value;
	},
	get showWhitespaceNodes() {
		return this.domview.showWhitespaceNodes;
	},
	set showWhitespaceNodes(value) {
		return this.domview.showWhitespaceNodes = value;
	},
	get whatToShow() {
		return this.domview.whatToShow;
	},
	set whatToShow(value) {
		return this.domview.whatToShow = value;
	},

	getCellText : function(row,column){
		return this.domview.getCellText(row, column);
	},
	setTree: function(treebox) {
		this.treebox = treebox;
		this.domview.setTree(treebox);
	},
	isContainer: function(row) {
		return this.domview.isContainer(row);
	},
	isContainerEmpty: function(row) {
		return this.domview.isContainerEmpty(row);
	},
	isSeparator: function(row) {
		return this.domview.isSeparator(row);
	},
	isSorted: function(row) {
		return this.domview.isSorted(row);
	},
	getLevel: function(row) {
		return this.domview.getLevel(row);
	},
	getImageSrc: function(row,col) {
		src = this.domview.getImageSrc(row,col);
		// Hook
		// Not using this, leaving it for CSS
		return src;
	},
	getRowProperties: function(row,props) {
		this.domview.getRowProperties(row,props);
		// Hook
		// Prefered marking method. Leaves it to CSS to actually style it.
		var node = this.getNodeFromRowIndex(row);
		if (node.xpathIsNodeSelected) {
			var aserv = XPCU.getService(AtomServiceCID, 'nsIAtomService');
			props.AppendElement(aserv.getAtom("xpathSelected"));
		}
	},
	getCellProperties: function(row,col,props) {
		this.domview.getCellProperties(row,col,props);
		// Hook
		// Prefered marking method. Leaves it to CSS to actually style it.
		
		// col is a string containing the ID of the column.
		
		// Return the same thing as getRowProperties() when dealing with the primary column
		var primaryCol = this.treebox.getColumnID(this.treebox.getKeyColumnIndex());
		var node = this.getNodeFromRowIndex(row);
		if (node.xpathIsNodeSelected && col == primaryCol) {
			var aserv = XPCU.getService(AtomServiceCID, 'nsIAtomService');
			props.AppendElement(aserv.getAtom("xpathSelected"));
		}
	},
	getColumnProperties: function(colid,col,props) {
		this.domview.getColumnProperties(colid,col,props);
	},
	canDrop: function( index , orientation ) {
		return this.domview.canDrop(index , orientation );
	},
	cycleCell: function( row , col ) {
		this.domview.cycleCell(row, col);
	},
	cycleHeader: function( col ) {
		this.domview.cycleCell(col);
	},
	drop: function( row , orientation ) {
		this.domview.cycleCell(row, orientation);
	},
	getCellText: function( row , col ) {
		return this.domview.getCellText(row , col);
	},
	getCellValue: function( row , col ) {
		return this.domview.getCellValue(row , col);
	},
	getNodeFromRowIndex: function( row ) {
		return this.domview.getNodeFromRowIndex(row);
	},
	getParentIndex: function( row ) {
		return this.domview.getParentIndex(row);
	},
	getProgressMode: function( row, col ) {
		return this.domview.getParentIndex(row, col);
	},
	getRowIndexFromNode: function( node ) {
		return this.domview.getRowIndexFromNode(node);
	},
	hasNextSibling: function( rowIndex, afterIndex ) {
		return this.domview.hasNextSibling(rowIndex, afterIndex);
	},
	isContainerOpen: function( index ) {
		return this.domview.isContainerOpen(index);
	},
	isEditable: function( row, col ) {
		return this.domview.isEditable(row, col);
	},
	performAction: function( action ) {
		this.domview.performAction(action);
		// Hook
		if (action == "xpath:clear") {
			// clear the current XPath selection
			for (i in this.selectedNodes) {
				node = this.selectedNodes[i];
				node.xpathIsNodeSelected = false;
				this.treebox.invalidateRow(this.getRowIndexFromNode(node));
			}
		}
	},
	performActionOnCell: function( action, row, col ) {
		this.domview.performActionOnCell(action, row, col);
		// Hook
	},
	performActionOnRow: function( action, row ) {
		this.domview.performActionOnRow(action, row);
		// Hook
		if (action == "xpath:select") {
			// Perform an XPath select
			var node = this.getNodeFromRowIndex(row);
			this.performActionOnNode("xpath:select", node);
		}
	},
	// A custom method. Performs a named action on the given node in the tree.
	performActionOnNode: function( action, node ) {
		// Hook
		if (action == "xpath:select") {
			// Perform an XPath select
			node.xpathIsNodeSelected = true;
			this.selectedNodes.push(node);
			this.treebox.invalidateRow(this.getRowIndexFromNode(node));
		}
	},
	rebuild: function() {
		// Clear data here
		this.performAction("xpath:clear");
		this.domview.rebuild();
	},
	selectionChanged: function() {
		this.domview.selectionChanged();
	},
	setCellText: function(row, col, value) {
		this.domview.setCellText(row, col, value);
	},
	setCellValue: function(row, col, value) {
		this.domview.setCellValue(row, col, value);
	},
	toggleOpenState: function( index ) {
		this.domview.toggleOpenState(index);
	}
};

